/*******************************************************************
 * File - bcuser.c 
 *
 * This is a simple test program that demonstrates the operation
 * of the different functions of the bc635/7PCI-V2/PCIe Time & Frequency 
 * Module using the device driver.
 * The code is written in such a way that the user can cut and paste
 * parts of the program to their custom application.
 *
 * This file may not be distributed -- it may only be used for 
 * development or evaluation purposes.
 * For details refer to SW_License.txt.
 *
 * Copyright (c) Symmetricom, 2000-2013
 *
 *******************************************************************/

#include "bcuser.h"
#include <sys/time.h>
#include <sys/select.h>

//**************************************************************************************
// Global variables and GPS Packet data structures
//**************************************************************************************
BYTE tmfmt, IntrUsed;
char rbuf[80];
char datbuf[80];

// GPS Time Packet 0x41
struct {			
	float	time;		
	short	week;	
	float	offset;	
} pac41;

// GPS Position Packet 0x42
struct {				
	float	X;		
	float	Y;	
	float	Z;
	float	TofF;
} pac42;

//**************************************************************************************
// Local functions
//**************************************************************************************
int   demo_menu(BC_PCI_HANDLE hBC_PCI);
int   errin(void);
int   errout(void);
int   interrupterr(void);
int   nxt(void);
int   usererr(void);
int   kbhit(void);
short getShort (char *pbDat);
float getFloat (char *pbDat);


//**************************************************************************************
// Description : main()
//**************************************************************************************
int main(int argc, char *argv[])
{
	BC_PCI_HANDLE hBC_PCI;

    // Start the device 
    hBC_PCI = bcStartPci();
    if (!hBC_PCI){
		printf ("Error Opening Device Driver\n");
		return 0;
    }

	// Init the time format
	if ( bcReqTimeFormat (hBC_PCI, &tmfmt) != TRUE )
		printf ("Error Getting Time Format!!!\n");

	// Show demo menu
	demo_menu(hBC_PCI);

	// Stop the device
	bcStopPci(hBC_PCI);

    return 0;
}

//**************************************************************************************
// Description : Read Time in Binary/Decimal
//**************************************************************************************
void pci_read_time (BC_PCI_HANDLE hBC_PCI)
{
	DWORD maj, min;
	WORD nano;
	BYTE stat;
	struct tm *majtime;
        struct tm dectime;

	if (tmfmt == FORMAT_DECIMAL)
	{  	
		// Decimal Time Format
		do{
			if ( bcReadDecTimeEx (hBC_PCI, &dectime, &min, &nano, &stat) == FALSE ){
				errout();
				break;
			}
	
			printf( "\nDecimal Time: %d %d  %02d:%02d:%02d.%06lu%d   Status: %d", 
					
                                        dectime.tm_yday, dectime.tm_year+1900,
					dectime.tm_hour, dectime.tm_min, dectime.tm_sec, min, nano, stat);
			
			bcDelay(10000);
		
		}while(!kbhit());
	} 
	else 
	{
		// Binary Time Format
		do{
			if ( bcReadBinTimeEx (hBC_PCI, &maj, &min, &nano, &stat) == FALSE ){
				errout();
				break;
			}

			majtime = gmtime( (time_t *)&maj );	
			printf( "\nBinary Time: %02d/%02d/%d  %02d:%02d:%02d.%06lu%d   Status: %d", 
					majtime->tm_mon+1, majtime->tm_mday, majtime->tm_year+1900,
					majtime->tm_hour, majtime->tm_min, majtime->tm_sec, min, nano, stat);
			bcDelay(10000);

		}while(!kbhit());
	}
}

//**************************************************************************************
// Description : Read Event Time 
//**************************************************************************************
void pci_read_event_time(BC_PCI_HANDLE hBC_PCI)
{
	DWORD evtmaj, evtmin;
	DWORD evtmajtmp=0, evtmintmp=0; 
	WORD evtnano;
	BYTE stat;
	struct tm *evtmajtm;

	do {
		if ( bcReadEventTimeEx (hBC_PCI, &evtmaj, &evtmin, &evtnano, &stat) == TRUE ){
			if ( evtmaj != evtmajtmp || evtmin != evtmintmp ){
				evtmajtm = gmtime( (time_t *)&evtmaj );

				printf( "\nTime: %02d/%02d/%d  %02d:%02d:%02d.%06lu%d   Status: %d", 
						evtmajtm->tm_mon+1, evtmajtm->tm_mday, evtmajtm->tm_year+1900,
						evtmajtm->tm_hour, evtmajtm->tm_min, evtmajtm->tm_sec, evtmin, evtnano, stat);
				
				// old event equal new event for next run
				evtmajtmp = evtmaj;
				evtmintmp = evtmin;
			}
		}
		bcDelay(1000);		
	}while(!kbhit());
}

//**************************************************************************************
// Description : Read Event2 Time 
//**************************************************************************************
void pci_read_event2_time(BC_PCI_HANDLE hBC_PCI)
{
	DWORD evtmaj, evtmin;
	DWORD evtmajtmp=0, evtmintmp=0; 
	WORD evtnano;
	BYTE stat;
	struct tm *evtmajtm;

	do {
		if ( bcReadEvent2TimeEx (hBC_PCI, &evtmaj, &evtmin, &evtnano, &stat) == TRUE ){
			if ( evtmaj != evtmajtmp || evtmin != evtmintmp ){
				evtmajtm = gmtime( (time_t *)&evtmaj );

				printf( "\nTime: %02d/%02d/%d  %02d:%02d:%02d.%06lu%d   Status: %d", 
						evtmajtm->tm_mon+1, evtmajtm->tm_mday, evtmajtm->tm_year+1900,
						evtmajtm->tm_hour, evtmajtm->tm_min, evtmajtm->tm_sec, evtmin, evtnano, stat);
				
				// old event equal new event for next run
				evtmajtmp = evtmaj;
				evtmintmp = evtmin;
			}
		}
                bcDelay(1000);		
	}while(!kbhit());
}

//**************************************************************************************
// Description : Read Event3 Time 
//**************************************************************************************
void pci_read_event3_time(BC_PCI_HANDLE hBC_PCI)
{
	DWORD evtmaj, evtmin;
	DWORD evtmajtmp=0, evtmintmp=0; 
	WORD evtnano;
	BYTE stat;
	struct tm *evtmajtm;

	do {
		if ( bcReadEvent3TimeEx (hBC_PCI, &evtmaj, &evtmin, &evtnano, &stat) == TRUE ){
			if ( evtmaj != evtmajtmp || evtmin != evtmintmp ){
				evtmajtm = gmtime( (time_t *)&evtmaj );

				printf( "\nTime: %02d/%02d/%d  %02d:%02d:%02d.%06lu%d   Status: %d", 
						evtmajtm->tm_mon+1, evtmajtm->tm_mday, evtmajtm->tm_year+1900,
						evtmajtm->tm_hour, evtmajtm->tm_min, evtmajtm->tm_sec, evtmin, evtnano, stat);
				
				// old event equal new event for next run
				evtmajtmp = evtmaj;
				evtmintmp = evtmin;
			}
		}
                bcDelay(1000);		
	}while(!kbhit());
}

//**************************************************************************************
// Description : Set Time
//**************************************************************************************
void pci_set_time (BC_PCI_HANDLE hBC_PCI)
{
	struct tm stm;
        struct tm *dec;
	int styear, stmon, stday, sthour, stmin, stsec;
	DWORD tm_sec;

	printf ("\n\n");
	printf("Enter Time:\n\nYYYY MM DD HH MM SS\n");
	scanf("%d %d %d %d %d %d", &styear, &stmon, &stday, &sthour, &stmin, &stsec);

	stm.tm_year = styear - 1900;
	stm.tm_mon	= stmon - 1;
	stm.tm_mday	= stday;
	stm.tm_hour = sthour;
	stm.tm_min	= stmin;
	stm.tm_sec	= stsec;
	stm.tm_isdst    = 0;
	tm_sec = mktime (&stm);
        
	if (tmfmt == FORMAT_DECIMAL)
	{  	
		dec = gmtime( (time_t *)&tm_sec );
		stm.tm_yday = dec->tm_yday + 1;			
		if ( bcSetDecTime (hBC_PCI, stm) == FALSE )
			errin ();

	}
	else
	{
		if ( bcSetBinTime (hBC_PCI, tm_sec) == FALSE )
			errin ();
	}
}

//**************************************************************************************
// Description : Set Year
//**************************************************************************************
void pci_set_year (BC_PCI_HANDLE hBC_PCI)
{
	int year;

	printf ("\n\n");
	printf ("Enter Year (1990 - 2036): ");
	scanf ("%d", &year);
	
	if ( bcSetYear (hBC_PCI, year) == FALSE )
		errin ();
}

//**************************************************************************************
// Description: Set Timing Mode
//**************************************************************************************
void pci_mode(BC_PCI_HANDLE hBC_PCI)
{
	int cmd;

	printf ("\n\n");
	printf ("0. Time Code Mode				\n");
	printf ("1. Free Running Mode			\n");
	printf ("2. 1PPS Mode					\n");
	printf ("3. RTC Mode					\n");
	printf ("4. GPS Mode					\n");
	printf ("9. Cancel - back to main menu	\n");
	printf ("   Select: ");
	cmd = 0;
    scanf ("%d",&cmd);

    switch(cmd){
	case 0:
		bcSetMode (hBC_PCI, MODE_IRIG);
		break;
	case 1:
		bcSetMode (hBC_PCI, MODE_FREE);
		break;
	case 2:
		bcSetMode (hBC_PCI, MODE_1PPS);
		break;		
	case 3:
		bcSetMode (hBC_PCI, MODE_RTC);	
		break;
	case 4:
		bcSetMode (hBC_PCI, MODE_GPS);
		break;
	case 9:	
		break;
	default:
		usererr ();
	}        
}

//**************************************************************************************
// Description: Read/Write Control Register
//**************************************************************************************
void pci_set_control(BC_PCI_HANDLE hBC_PCI)
{
	DWORD dwData;
        int   data;

	if ( bcReadReg  (hBC_PCI, PCI_CONTROL_OFFSET, &dwData) == TRUE )
	{
		printf ("\n\n");
		printf ("Current Control Register Value: %x\n", (int)dwData);
		printf ("Enter Hex Value for Control Register: ");
		scanf ("%x", &data);
				
		if ( bcWriteReg  (hBC_PCI, PCI_CONTROL_OFFSET, (DWORD)data) == FALSE )
				errin();
				
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set Local Time Offset
//**************************************************************************************
void pci_set_loc_off (BC_PCI_HANDLE hBC_PCI)
{
	int data;
	USHORT tmoff = 0;
	BYTE half = 0;

	// Make sure we enable the local time offset flag first
	bcSetLocalOffsetFlag(hBC_PCI, LOCAL_OFF_ENABLE);

	printf ("\n\n");
	printf ("Enter Local Time Offset (-16 to 16): ");
	scanf("%d", &data);
	tmoff = (USHORT)data & 0xffff;
	printf ("\nAdd Half Hour?\n 0: No\n 1: Yes\nSelect: ");
	scanf("%d", &data);
	half = (BYTE)data & 0xff;
	
	if ( bcSetLocOff (hBC_PCI, tmoff, half) == FALSE )
		errin();
}

//**************************************************************************************
// Description : Set Generator Time Offset
//**************************************************************************************
void pci_set_gen_off (BC_PCI_HANDLE hBC_PCI)
{
	int data;
	USHORT genoff = 0;
	BYTE half = 0;

	printf ("\n\n");
	printf ("Enter Generator Time Offset (-16 to 16): ");
	scanf("%d", &data);
	genoff = (USHORT)data & 0xffff;
	
	printf ("\nAdd Half Hour?\n 0: No\n 1: Yes\nSelect: ");
	scanf("%d", &data);
	half = (BYTE)data & 0xff;

	if ( bcSetGenOff (hBC_PCI, genoff, half) == FALSE )
		errin();

}

//**************************************************************************************
// Description : Set Strobe Time
//**************************************************************************************
void pci_set_strobe (BC_PCI_HANDLE hBC_PCI)
{
	int strbmode = 0;
	struct tm stm;
	int styear, stmon, stday, sthour, stmin, stsec;
	DWORD tm_sec = 0;
	DWORD tm_minor = 0;
	DWORD ctlreg;
	
	printf ("\n\nSelect Strobe Mode:\n\n");
	printf (" 1. Major and Minor\n");
	printf (" 2. Minor Only\n");
	printf (" 0. Cancel\n\n");
	printf ("    Select: ");
	scanf  ("%d", &strbmode);

	if (strbmode != 0)
	{
		if ( bcReadReg  (hBC_PCI, PCI_CONTROL_OFFSET, &ctlreg) == TRUE )
			ctlreg |= 0x10;
		else
			errout();
	}

	// major and minor
	if (strbmode == 1)
	{
		ctlreg &= 0xDF;
		if ( bcWriteReg  (hBC_PCI, PCI_CONTROL_OFFSET, ctlreg) == TRUE ){
			printf("\n\nEnter Major Time:\n\nYYYY MM DD HH MM SS\n");
			scanf("%d %d %d %d %d %d", &styear, &stmon, &stday,
											&sthour, &stmin, &stsec);
			
			stm.tm_year = styear - 1900;
			stm.tm_mon	= stmon - 1;
			stm.tm_mday	= stday;
			stm.tm_hour = sthour;
			stm.tm_min	= stmin;
			stm.tm_sec	= stsec;
                        stm.tm_isdst    = 0;
			tm_sec = mktime (&stm);
			
			printf("\nEnter Minor time: ");
			scanf("%lu", &tm_minor);
			
			if ( bcSetStrobeTime (hBC_PCI, tm_sec, tm_minor) == FALSE)
				errin ();
		}
		else
			errin();
	}
	else if (strbmode == 2)
	{
		ctlreg |= 0x20;
		if ( bcWriteReg  (hBC_PCI, PCI_CONTROL_OFFSET, ctlreg) == TRUE){
			printf("\nEnter minor time: ");
			scanf("%lu", &tm_minor);
		
			if ( bcSetStrobeTime (hBC_PCI, tm_sec, tm_minor) == FALSE)
				errin ();
		}
		else
			errin();		
	}
}


//**************************************************************************************
// Description : Set Time Code Format & Type
//**************************************************************************************
void pci_time_code (BC_PCI_HANDLE hBC_PCI)
{
	int tc_fmt;
	int tc_mod;

	printf ("\n\nSelect Time Code Format:\n\n");
	printf (" 1. IRIG A\n");
	printf (" 2. IRIG B\n");
	printf (" 3. IEEE 1344\n");
	printf (" 4. NASA 36\n\n");
	printf ("    Select: ");
	scanf  ("%d", &tc_fmt);
	switch ( tc_fmt )
	{
		case 1: if (bcSetTcIn(hBC_PCI,TCODE_IRIG_A) == FALSE)
					errin();
				break;
		case 2: if (bcSetTcIn(hBC_PCI,TCODE_IRIG_B) == FALSE)
					errin();			
				break;
		case 3: if (bcSetTcIn(hBC_PCI,TCODE_IEEE) == FALSE)
					errin();			
				break;
		case 4: if (bcSetTcIn(hBC_PCI,TCODE_NASA) == FALSE)
					errin();			
				break;
		default:usererr();
				break;
	}

	printf ("\n\nSelect Time Code Modulation:\n\n");
	printf (" 1. Modulated\n");
	printf (" 2. DC Level Shift\n\n");
	printf ("    Select: ");
	scanf  ("%d", &tc_mod);
	switch ( tc_mod )
	{
		case 1: if (bcSetTcInMod(hBC_PCI,TCODE_MOD_AM) == FALSE)
					errin();			
				break;
		case 2: if (bcSetTcInMod(hBC_PCI,TCODE_MOD_DC) == FALSE)
					errin();		
				break;
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Set Output Frequency
//**************************************************************************************
void pci_out_freq (BC_PCI_HANDLE hBC_PCI)
{
    DWORD ctlreg;
	int out_freq = 0;

	if ( bcReadReg  (hBC_PCI, PCI_CONTROL_OFFSET, &ctlreg) == TRUE )
	{
		ctlreg &= 0x3F;
	
		printf ("\n\nSelect Output Frequency:\n\n");
		printf (" 1. 1MHz\n 2. 5MHz\n 3. 10MHz\n\n");
		printf (" Select: ");
		scanf  ("%d", &out_freq);
		switch ( out_freq )
		{
			case 1: ctlreg |= 0x80;
					break;
			case 2: ctlreg |= 0x40;		
					break;
			case 3: 
					break;
			default:usererr();
					break;
		}
		
		if (bcWriteReg (hBC_PCI, PCI_CONTROL_OFFSET, ctlreg) == FALSE)
			errin();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set Time Format
//**************************************************************************************
void pci_time_format(BC_PCI_HANDLE hBC_PCI)
{
	int time_format = 0;
	
	printf ("\n\nSelect Time Format\n  0. Decimal\n  1. Binary\n\nSelect: ");
	scanf ("%d", &time_format);
	
	switch ( time_format )
	{
		case 0: if (bcSetTimeFormat(hBC_PCI,FORMAT_DECIMAL) == TRUE)
					tmfmt = FORMAT_DECIMAL;
				else
					errin();
				break;
		case 1: if (bcSetTimeFormat(hBC_PCI,FORMAT_BINARY) == TRUE)
					tmfmt = FORMAT_BINARY;
				else
					errin();
				break;				
		default:usererr();
				break;
	}
}
	
//**************************************************************************************
// Description : Set Periodic (Heartbeat) Output
//**************************************************************************************
void pci_heartbeat (BC_PCI_HANDLE hBC_PCI)
{
	int  hrt_mode;
	int  counter1 = 10; 
	int  counter2 = 10;
	BYTE mode = PERIODIC_SYNC;
	USHORT n1, n2;
        n1 = n2 = 1000;

	printf ("\n\nHeartbeat Counters and Mode:\n\n");
	printf (" 1. Synchronous mode\n");
	printf (" 2. Asynchronous mode\n\n");
	printf (" Select: ");
	scanf  ("%d", &hrt_mode);
	
	printf ("\n Enter decimal value for counter1: ");
	scanf  ("%d", &counter1);
	printf ("\n Enter decimal value for counter2: ");
	scanf  ("%d", &counter2);

	if (hrt_mode == 2)
		mode = PERIODIC_NOSYNC;

	if (counter1 >= 2 && counter1 <= 65535)
		n1 = (USHORT)counter1;

	if (counter2 >= 2 && counter2 <= 65535)
		n2 = (USHORT)counter2;
	
	if (bcSetHbt (hBC_PCI, mode, n1, n2) == FALSE)
		errin();	
}


//**************************************************************************************
// Description : Set Propagation Delay
//**************************************************************************************
void pci_set_prop_delay (BC_PCI_HANDLE hBC_PCI)
{
	long prop_delay = 0;

	printf("\n\nEnter Propagation Delay: ");
	scanf("%ld", &prop_delay);

	if ( bcSetPropDelay (hBC_PCI, prop_delay) == FALSE)
		errin();
	
}

//**************************************************************************************
// Description : Software Rest
//**************************************************************************************
void pci_sw_rest (BC_PCI_HANDLE hBC_PCI)
{
	int yesno;

	printf("\n\nReset the board?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf("%d", &yesno);

	if ( yesno == 1 ){
		bcCommand (hBC_PCI, CMD_WARMSTART);
		tmfmt = FORMAT_BINARY;
	}
}

//**************************************************************************************
// Description : Set TimeCode Output Format
//**************************************************************************************
void pci_tc_out_format(BC_PCI_HANDLE hBC_PCI)
{
	int tc_out = 0;
	
	printf ("\n\nSelect Time Code Output Format\n  0. IRIG B\n  1. IEEE 1344\n\nSelect: ");
	scanf ("%d", &tc_out);
	
	switch ( tc_out )
	{
		case 0: if (bcSetGenCode(hBC_PCI,TCODE_IRIG_B) == FALSE)
					errin();
				break;
		case 1: if (bcSetGenCode(hBC_PCI,TCODE_IEEE) == FALSE)
					errin();
				break;
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Set Clock Source
//**************************************************************************************
void pci_set_clock_src(BC_PCI_HANDLE hBC_PCI)
{
	int clk_src = 0;
	
	printf ("\n\nSelect Clock Source\n  0. Internal\n  1. External\n\nSelect: ");
	scanf ("%d", &clk_src);
	
	switch ( clk_src )
	{
		case 0: if (bcSetClkSrc(hBC_PCI,CLK_INT) == FALSE)
					errin();
				break;
		case 1: if (bcSetClkSrc(hBC_PCI,CLK_EXT) == FALSE)
					errin();
				break;				
		default:usererr();
				break;
	}
}


//**************************************************************************************
// Description : Program Leap Seconds
//**************************************************************************************
void pci_set_leap_sec (BC_PCI_HANDLE hBC_PCI)
{
	int	leaptm = 0;
	int     dummy = 0;			
	char	leapflag;

	printf ("\n\nEnter Leap Event Time in Seconds (Unix Time): ");
	scanf  ("%d", &leaptm);

	printf ("\nLeap Second Flag?\n  0: Disable\n  1: Insertion\n -1: Deletion\nSelect: ");
	scanf ("%d", &dummy);
	
	leapflag = (char)dummy & 0xff;

	if ( bcSetLeapEvent (hBC_PCI, leapflag, (DWORD)leaptm) == FALSE)
		errin();

}      
        
//**************************************************************************************
// Description : Control Jam-Sync
//**************************************************************************************
void pci_ctl_jam_sync(BC_PCI_HANDLE hBC_PCI)
{
	int jam_sync = 1;
	
	printf ("\n\nControl Jam-Sync\n 0. Disable Jam-Sync\n 1. Enable Jam-Sync\n\nSelect: ");
	scanf ("%d", &jam_sync);
	
	switch ( jam_sync )
	{
		case 0: if (bcSetJam(hBC_PCI,JAM_SYNC_DIS) == FALSE)
					errin();
				break;
		case 1: if (bcSetJam(hBC_PCI,JAM_SYNC_ENA) == FALSE)
					errin();
				break;				
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Force Jam-Sync
//**************************************************************************************
void pci_frc_jam_sync(BC_PCI_HANDLE hBC_PCI)
{
	int jam_sync = 0;
	
	printf ("\n\nForce Jam-Sync?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf ("%d", &jam_sync);
	
	if (jam_sync == 1){
		if ( bcForceJam (hBC_PCI) == FALSE)
			errin();
	}
}


//**************************************************************************************
// Description : Load D/A Converter
//**************************************************************************************
void pci_set_da_con(BC_PCI_HANDLE hBC_PCI)
{
	int da_con = 0;
	
	printf ("\n\nEnter D/A value (0 to 65535): ");
	scanf ("%d", &da_con);
	
	if ( bcSetDac (hBC_PCI, (USHORT)da_con) == FALSE )
		errin();
}

//**************************************************************************************
// Description : Set Disciplining Gain
//**************************************************************************************
void pci_set_gain(BC_PCI_HANDLE hBC_PCI)
{
	short dis_gain = 0;
	
	printf ("\n\nEnter Gain value (-32768 to 32767): ");
	scanf ("%hd", &dis_gain);
	
	if ( bcSetGain (hBC_PCI, dis_gain) == FALSE )
		errin();
}

//**************************************************************************************
// Description : Sync RTC to External Time Data
//**************************************************************************************
void pci_sync_rtc(BC_PCI_HANDLE hBC_PCI)
{
	int sync_rtc = 0;
	
	printf ("\n\nSync RTC to External Time Data?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf ("%d", &sync_rtc);
	
	if ( sync_rtc ){
		if ( bcSyncRtc (hBC_PCI) == FALSE )
			errin();
	}
}

//**************************************************************************************
// Description : Disconnect Battery to RTC Chip
//**************************************************************************************
void pci_dis_rtc(BC_PCI_HANDLE hBC_PCI)
{
	int dis_rtc = 0;
	
	printf ("\n\nDisconnect Battery to RTC Chip?\n 0. No\n 1. Yes\n\nSelect: ");
	scanf ("%d", &dis_rtc);
	
	if ( dis_rtc ){
		if ( bcDisRtcBatt (hBC_PCI) == FALSE )
			errin();
	}
}

//**************************************************************************************
// Description : Set Clock Value
//**************************************************************************************
void pci_set_clk_val(BC_PCI_HANDLE hBC_PCI)
{
	long clk_val = 0;
	
	printf ("\n\nEnter Clock Value: ");
	scanf ("%ld", &clk_val);
	
	if ( bcAdjustClock (hBC_PCI, clk_val) == FALSE )
		errin();
}


//**************************************************************************************
// Description : Set GPS Time Format
//**************************************************************************************
void pci_set_gps_tm_fmt(BC_PCI_HANDLE hBC_PCI)
{
	int gps_tm_fmt = 0;
	
	printf ("\n\nGPS Time Format\n 0. UTC Time\n 1. GPS Time\n\nSelect: ");
	scanf ("%d", &gps_tm_fmt);
	
	switch ( gps_tm_fmt )
	{
		case 0: if ( bcSetGpsTmFmt (hBC_PCI, UTC_TIME_FMT) == FALSE )
					errin();
				break;
		case 1: if ( bcSetGpsTmFmt (hBC_PCI, GPS_TIME_FMT) == FALSE )
					errin();		
				break;				
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Set GPS Mode Flag
//**************************************************************************************
void pci_set_gps_mode_flg(BC_PCI_HANDLE hBC_PCI)
{
	int gps_mode_flg = 1;
	
	printf ("\n\nGPS Mode Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
	scanf ("%d", &gps_mode_flg);
	
	switch ( gps_mode_flg )
	{
		case 0: if ( bcSetGpsOperMode(hBC_PCI, GPS_NONE_STATIC) == FALSE )
					errin();
				break;
		case 1: if ( bcSetGpsOperMode(hBC_PCI, GPS_STATIC) == FALSE )
					errin();
				break;		
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Set Local Time Offset Flag
//**************************************************************************************
void pci_set_local_off_flg(BC_PCI_HANDLE hBC_PCI)
{
	int local_off_flg = 1;
	
	printf ("\n\nLocal Time Offset Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
	scanf ("%d", &local_off_flg);
	
	switch ( local_off_flg )
	{
		case 0: if ( bcSetLocalOffsetFlag(hBC_PCI, LOCAL_OFF_DISABLE) == FALSE )
					errin();
				break;
		case 1: if ( bcSetLocalOffsetFlag(hBC_PCI, LOCAL_OFF_ENABLE) == FALSE )
					errin();
				break;			
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Set Year Auto Increment Flag
//**************************************************************************************
void pci_set_yr_auto_inc_flg(BC_PCI_HANDLE hBC_PCI)
{
	int yr_auto_flg = 1;
	
	printf ("\n\nYear Auto Increment Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
	scanf ("%d", &yr_auto_flg);
	
	switch ( yr_auto_flg )
	{
		case 0: if ( bcSetYearAutoIncFlag(hBC_PCI, YEAR_AUTO_DIS) == FALSE )
					errin();
				break;
		case 1: if ( bcSetYearAutoIncFlag(hBC_PCI, YEAR_AUTO_ENA) == FALSE )
					errin();
				break;				
		default:usererr();
				break;
	}
}

//**************************************************************************************
// Description : Request Time Settings
//**************************************************************************************
void pci_req_time_settings (BC_PCI_HANDLE hBC_PCI)
{
	TimeData TimeDat;
	const char smode[7][10] = {"Time Code","Free Run","Ext 1PPS","RTC","Reserved","Reserved","GPS"};
	const char stm_fmt[2][10] = {"Decimal","Binary"};
	const char flags[2][10] = {"Disable","Enable"};
	const char gps_utc[2][12] = {"UTC Format","GPS Format"};
	float loc_off;

	if ( bcReqTimeData(hBC_PCI, &TimeDat) == TRUE )
	{
		printf("\n\nTime Settings:\n");
		printf("\n Mode\t\t\t\t: %s",smode[TimeDat.mode]);
		printf("\n Time Format\t\t\t: %s",stm_fmt[TimeDat.tmformat]);
		printf("\n Year\t\t\t\t: %d",(int)TimeDat.year);
		
		loc_off = (float)TimeDat.locoff;
		if (TimeDat.lhlfhr)
			loc_off += 0.5;
		
		printf("\n Local Offset     \t\t: %.1f",loc_off);
		printf("\n Propagation Delay\t\t: %d", (int)TimeDat.propdelay);
		printf("\n Current Leap Seconds\t\t: %d", TimeDat.leapsec);
		printf("\n Scheduled Leap Event Time\t: %ld", TimeDat.leapevttm);

		if (TimeDat.leapevt == -1)
			printf("\n Scheduled Leap Event Flag\t: Deletion");
		else if (TimeDat.leapevt == 0)
			printf("\n Scheduled Leap Event Flag\t: Disable");
		else if (TimeDat.leapevt == 1)
			printf("\n Scheduled Leap Event Flag\t: Insertion");
		else
			printf("\n Scheduled Leap Event Flag\t: Unknown");
		
		printf("\n GPS Time Format\t\t: %s", gps_utc[TimeDat.gpstmfmt]);
		printf("\n IEEE Daylight Savings Flag\t: %s", flags[TimeDat.dlight]);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Request Time Code Settings
//**************************************************************************************
void pci_req_timecode_settings (BC_PCI_HANDLE hBC_PCI)
{
	static char szTcOutIRIGA[8][36] = {
		"IRIG A000, A130 (BCD,CF,SBS)",
		"IRIG A001, A131 (BCD,CF)",
		"IRIG A002, A132 (BCD)",
		"IRIG A003, A133 (BCD,SBS)",
		"IRIG A004, A134 (BCD,YR,CF,SBS)",
		"IRIG A005, A135 (BCD,YR,CF)",
		"IRIG A006, A136 (BCD,YR)",
		"IRIG A007, A137 (BCD,YR,SBS)",
	};
	static char szTCOutIRIGB[8][36] = {
		"IRIG B000, B120 (BCD,CF,SBS)",
		"IRIG B001, B121 (BCD,CF)",
		"IRIG B002, B122 (BCD)",
		"IRIG B003, B123 (BCD,SBS)",
		"IRIG B004, B124 (BCD,YR,CF,SBS)",
		"IRIG B005, B125 (BCD,YR,CF)",
		"IRIG B006, B126 (BCD,YR)",
		"IRIG B007, B127 (BCD,YR,SBS)",
	};
	TimeCodeDataEx TimeCodeDat;
	float gen_off;

	if ( bcReqTimeCodeDataEx(hBC_PCI, &TimeCodeDat) == TRUE )
	{
		printf("\n\nTime Code Settings:\n");

		switch (TimeCodeDat.inputFormat) {
		case TCODE_IRIG_A:
			if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_NONE)
				printf("\n Input Time Code\t: IRIG A000-A003, A130-A133 (no year)" );
			else if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_Y)
				printf("\n Input Time Code\t: IRIG A004-A007, A134-A137 (has year)" );
			else
				printf("\n Input Time Code\t: Unknown" );
			break;
		case TCODE_IRIG_B:
			if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_NONE)
				printf("\n Input Time Code\t: IRIG B000-B003, B120-B123 (no year)" );
			else if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_Y)
				printf("\n Input Time Code\t: IRIG B004-B007, B124-B127 (has year)" );
			else if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_T)
				printf("\n Input Time Code\t: IRIG B TrueTime Legacy" );
			else
				printf("\n Input Time Code\t: Unknown" );
			break;
		case TCODE_IEEE:
			printf("\n Input Time Code\t: IRIG B IEEE 1344" );
			break;
		case TCODE_IRIG_E:
			if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_NONE)
				printf("\n Input Time Code\t: IRIG E001, E002, E121, E122 (no year)" );
			else if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_Y)
				printf("\n Input Time Code\t: IRIG E005, E006, E125, E126 (has year)" );
			else
				printf("\n Input Time Code\t: Unknown" );
			break;
		case TCODE_IRIG_e:
			if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_NONE)
				printf("\n Input Time Code\t: IRIG E001, E002, E111, E112 (no year)" );
			else if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_Y)
				printf("\n Input Time Code\t: IRIG E005, E006, E115, E116 (has year)" );
			else
				printf("\n Input Time Code\t: Unknown" );
			break;
		case TCODE_IRIG_G:
			if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_NONE)
				printf("\n Input Time Code\t: IRIG G001, G002, G141, G142 (no year)" );
			else if (TimeCodeDat.inputSubType == TCODE_IRIG_SUBTYPE_Y)
				printf("\n Input Time Code\t: IRIG G005, G006, G145, E146 (has year)" );
			else
				printf("\n Input Time Code\t: Unknown" );
			break;
		case TCODE_NASA:
			printf("\n Input Time Code\t: NASA 36" );
			break;
		case TCODE_XR3:
			printf("\n Input Time Code\t: XR3" );
			break;
		case TCODE_2137:
			printf("\n Input Time Code\t: 2137" );
			break;
		default:
			printf("\n Input Time Code\t: Unknown" );
			break;
		}

		switch (TimeCodeDat.modulation){
		case TCODE_MOD_AM:
			printf("\n Code Modulation\t: AM");
			break;
		case TCODE_MOD_DC:
			printf("\n Code Modulation\t: DC");
			break;
		default:
			printf("\n Code Modulation\t: Unknown");
			break;
		}

		switch (TimeCodeDat.outputFormat){
		case TCODE_IRIG_A:
			if (TimeCodeDat.outputSubType >= TCODE_IRIG_SUBTYPE_0 &&
				TimeCodeDat.outputSubType <= TCODE_IRIG_SUBTYPE_7)
				printf("\n Output Time Code\t: %s", szTcOutIRIGA[TimeCodeDat.outputSubType-TCODE_IRIG_SUBTYPE_0]);
			else
				printf("\n Output Time Code\t: Unknown");
			break;
		case TCODE_IRIG_B:
			if (TimeCodeDat.outputSubType >= TCODE_IRIG_SUBTYPE_0 &&
				TimeCodeDat.outputSubType <= TCODE_IRIG_SUBTYPE_7)
				printf("\n Output Time Code\t: %s", szTCOutIRIGB[TimeCodeDat.outputSubType-TCODE_IRIG_SUBTYPE_0]);
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_T)
				printf("\n Output Time Code\t: IRIG B TrueTime (BCD,CF,SBS)");
			else
				printf("\n Output Time Code\t: Unknown");
			break;
		case TCODE_IEEE:
			printf("\n Output Time Code\t: IRIG B IEEE 1344");
			break;
		case TCODE_IRIG_E:
			if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_1)
				printf("\n Output Time Code\t: IRIG E001, E121 (BCD,CF)");
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_2)
				printf("\n Output Time Code\t: IRIG E002, E122 (BCD)");
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_5)
				printf("\n Output Time Code\t: IRIG E005, E125 (BCD,YR,CF)");
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_6)
				printf("\n Output Time Code\t: IRIG E006, E126 (BCD,YR)");
			else
				printf("\n Output Time Code\t: Unknown");
			break;
		case TCODE_IRIG_e:
			if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_1)
				printf("\n Output Time Code\t: IRIG E001, E111 (BCD,CF)");
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_2)
				printf("\n Output Time Code\t: IRIG E002, E112 (BCD)");
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_5)
				printf("\n Output Time Code\t: IRIG E005, E115 (BCD,YR,CF)");
			else if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_6)
				printf("\n Output Time Code\t: IRIG E006, E116 (BCD,YR)");
			else
				printf("\n Output Time Code\t: Unknown");
			break;
		case TCODE_IRIG_G:
			if (TimeCodeDat.outputSubType == TCODE_IRIG_SUBTYPE_5)
				printf("\n Output Time Code\t: IRIG G005, G145 (BCD,YR,CF)");
			else
				printf("\n Output Time Code\t: Unknown");
			break;
		case TCODE_NASA:
			printf("\n Output Time Code\t: NASA 36");
			break;
		case TCODE_XR3:
			printf("\n Output Time Code\t: XR3");
			break;
		case TCODE_2137:
			printf("\n Output Time Code\t: 2137");
			break;
		default:
			printf("\n Output Time Code\t: Unknown");
			break;
		}

		gen_off = (float)TimeCodeDat.genoffset;

		if (TimeCodeDat.ghfhr && gen_off >= 0)
			gen_off += 0.5;
		else if (TimeCodeDat.ghfhr && gen_off < 0)
			gen_off -= 0.5;

		printf("\n Generator Time Offset\t: %.1f",gen_off);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Request Clock Settings
//**************************************************************************************
void pci_req_clock_settings (BC_PCI_HANDLE hBC_PCI)
{
	OscData OscDat;
	const char flags[2][10] = {"Disable","Enable"};

	if ( bcReqOscData (hBC_PCI, &OscDat) == TRUE )
	{
		printf("\n\nClock Settings:\n");

		if (OscDat.ClkSrc == 'E')
			printf("\n Oscillator  \t: External");
		else
			printf("\n Oscillator  \t: Internal");
	
		printf("\n DAC Value\t: %d",OscDat.DacVal);
		printf("\n Jam Control\t: %s",flags[OscDat.JamCtl]);
		
		if (OscDat.BattStat == 0) 
			printf("\n Battery Status\t: Failed");
		else if (OscDat.BattStat == 0x10)
 			printf("\n Battery Status\t: Passed");
		else
			printf("\n Battery Status\t: Unknown");

		printf("\n Clock Value\t: %d",(int)OscDat.AdjClk);
		printf("\n Disc Control\t: %s",flags[OscDat.DisCtl]); 
		printf("\n Phase Control\t: %s",flags[OscDat.JamCtl]); 
		printf("\n Disc Gain\t: %d",OscDat.DisGain);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Request Control Settings
//**************************************************************************************
void pci_req_control_settings (BC_PCI_HANDLE hBC_PCI)
{
	OtherDataEx OtherDat;
	const char htbeat[2][13]    = {"Not Sync'd","Sync'd"};
	const char evtflag[2][10]   = {"Disable","Enable"};
	const char evtedge[2][10]   = {"Rising","Falling"};
	const char evtsource[2][10] = {"Event","Periodic"};

	if ( bcReqOtherDataEx (hBC_PCI, &OtherDat) == TRUE )
	{
		printf("\n\nControl Settings:\n");
		printf("\n HeartBeat Mode\t\t: %s",htbeat[OtherDat.hbtmode]);
		printf("\n HeartBeat Counter1\t: %d",OtherDat.hbtcnt1); 
		printf("\n HeartBeat Counter2\t: %d",OtherDat.hbtcnt2); 
		printf("\n Frequency Output\t: %d MHz", OtherDat.freq); 
		printf("\n Event Control\t\t: %s", evtflag[OtherDat.evtctl]);
		printf("\n Event Edge\t\t: %s", evtedge[OtherDat.evtsense]);
		printf("\n Event Capture Lockout\t: %s", evtflag[OtherDat.evtlock]);
		printf("\n Event Capture Source\t: %s", evtsource[OtherDat.evtsrc]);
		printf("\n Event2 Control\t\t: %s", evtflag[OtherDat.evt2ctl]);
		printf("\n Event2 Edge\t\t: %s", evtedge[OtherDat.evt2sense]);
		printf("\n Event2 Capture Lockout\t: %s", evtflag[OtherDat.evt2lock]);
		printf("\n Event3 Control\t\t: %s", evtflag[OtherDat.evt3ctl]);
		printf("\n Event3 Edge\t\t: %s", evtedge[OtherDat.evt3sense]);
		printf("\n Event3 Capture Lockout\t: %s", evtflag[OtherDat.evt3lock]);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Request Model, Serial Number, Assembly Number and Hardware FAB 
//**************************************************************************************
void pci_req_modal_data (BC_PCI_HANDLE hBC_PCI)
{
	ModelData ModelDat;
	DWORD serial;
	WORD fab, assemb;

	printf("\n\nModel Information\n");
	if ( bcReqModel (hBC_PCI, &ModelDat) == TRUE )
	{
		printf("\n Model\t\t: %c%c%c%c%c%c%c%c",
					ModelDat.model1, ModelDat.model2, ModelDat.model3,
					ModelDat.model4, ModelDat.model5, ModelDat.model6,
					ModelDat.model7, ModelDat.model8);

		if ( bcReqSerialNum (hBC_PCI, &serial) == TRUE )
			printf("\n Serial\t\t: %08X", (int)serial);
		else
			printf("\n Serial\t\t: Error Getting Data");

		if ( bcReqHardwarFab (hBC_PCI, &fab) == TRUE )
			printf("\n Hardware Fab\t: %d",fab);
		else
			printf("\n Hardware Fab\t: Error Getting Data");

		if ( bcReqAssembly (hBC_PCI, &assemb) == TRUE )
			printf("\n Assembly\t: %d",assemb);
		else
			printf("\n Assembly\t: Error Getting Data");

		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Request Firmware Version
//**************************************************************************************
void pci_req_fw_ver (BC_PCI_HANDLE hBC_PCI)
{
	VerData verdat;

	if ( bcReqVerData (hBC_PCI, &verdat) == TRUE )
	{
		printf ("\n\n");
		printf ("Firmware Version : %c%c%c%c%c%c%c%c%c%c%c\n",
				verdat.byt1, verdat.byt2, verdat.byt3, verdat.byt4,
				verdat.byt5, verdat.byt6, verdat.byt7, verdat.byt8,
				verdat.byt9, verdat.byt10, verdat.byt11);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set Interrupts and enable interrupt thread
//**************************************************************************************
void pci_set_ints (BC_PCI_HANDLE hBC_PCI)
{
	int cmd = 0;

	printf ("\n\n");
	printf ("Enter interrupt mask in two hex digits (example 0x02, 0x0a): 0x");
	scanf ("%x",&cmd);

	cmd &= 0x7F;

	if (bcStartIntEx(hBC_PCI, bcIntHandlerRoutine, cmd) == FALSE)
	{
		printf("\n\nError Starting Interrupt!\n");
		nxt();
		return;
	}

	if (bcSetInt(hBC_PCI, cmd) == FALSE)
		interrupterr();
}

//**************************************************************************************
// Description : Request Interrupts and enable interrupt thread
//**************************************************************************************
void pci_req_ints (BC_PCI_HANDLE hBC_PCI)
{
	BYTE IntUsed;
	

	if (bcReqInt(hBC_PCI,&IntUsed) == TRUE) {
		printf("\nInterrupt: 0x%02X", IntUsed);
	}
	else {
		errout();
	}
	
}

//**************************************************************************************
// Description : Request PCI Revision ID of the PCI configuration space
//**************************************************************************************
void pci_req_rev_id (BC_PCI_HANDLE hBC_PCI)
{
	WORD wRevID;

	if ( bcReqRevisionID (hBC_PCI, &wRevID) == TRUE )
	{
		printf ("\n\n");
		printf ("PCI Board Revision ID : %d (0x%2x)\n", (int)wRevID, (unsigned int)wRevID);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Show Interrupts
//**************************************************************************************
void bcIntHandlerRoutine(BC_PCI_HANDLE hBC_PCI, DWORD dwSource)
{
	DWORD maj, min;
	WORD nano;
	BYTE stat;
	struct tm *majtime;
	struct tm  dectime;

	if (dwSource < 0x01 || dwSource > 0x7F) {
		printf("\nError: Unknown Interrupt!!!");
		return;
	}

	if (dwSource & INTERRUPT_EVENT) {
		printf("Event | ");
	}

	if (dwSource & INTERRUPT_PERIODIC) {
		printf("Periodic | ");
	}

	if (dwSource & INTERRUPT_STROBE) {
		printf("Strobe | ");
	}

	if (dwSource & INTERRUPT_1PPS) {
		printf("1PPS | ");
	}

	if (dwSource & INTERRUPT_GPS) {
		printf("GPS | ");
	}

	if (dwSource & INTERRUPT_EVENT2) {
		printf("Event2 | ");
	}

	if (dwSource & INTERRUPT_EVENT3) {
		printf("Event3 | ");
	}

	if (tmfmt == FORMAT_DECIMAL) {

		bcReadDecTimeEx (hBC_PCI, &dectime, &min, &nano, &stat);
 
		printf("%02d:%02d:%02d.%06lu%d", 
			dectime.tm_hour, dectime.tm_min, dectime.tm_sec, min, nano);
	} 
	else  {

		bcReadBinTimeEx (hBC_PCI, &maj, &min, &nano, &stat);

		majtime = gmtime( (time_t *)&maj );
		printf( "%02d:%02d:%02d.%06lu%d", 
			majtime->tm_hour, majtime->tm_min, majtime->tm_sec, min, nano);
	}

	printf("\nPress 2 and Enter to stop interrupt routine!\n\n");
}


//**************************************************************************************
// Description : GPS Packet 41 - GPS Time Packet
//**************************************************************************************
void getPac41 (BC_PCI_HANDLE hBC_PCI)
{
	GpsPkt pktIn;
	GpsPkt pktOut;

	printf ("\n\nGPS PACKET 41 - GPS TIME PACKET\n\n");

	pktIn.id = 0x21;
	pktIn.len = 1;					
	pktIn.data = 0;
	pktOut.len = 10;				
	pktOut.id = 0x41;
	pktOut.data = (BYTE *) rbuf;

	if ( bcGPSMan (hBC_PCI, &pktIn, &pktOut) == TRUE )
	{	
		pac41.time   = getFloat (&rbuf[0]);
		pac41.week   = getShort (&rbuf[4]);
		pac41.offset = getFloat (&rbuf[6]);
		
		printf ("Seconds of Week: %0.2f\n", pac41.time);
		printf ("GPS Week Number: %hd\n", pac41.week);
		printf ("GPS/UCT Offset:  %0.2f\n\n", pac41.offset);
		
		nxt();
	}
	else
	{
		// Since the reciver may not have the data ready yet,
		// lets give it one more try
		sleep(1);
		pktIn.id = 0x21;
		pktIn.len = 1;					
		pktIn.data = 0;
		pktOut.len = 10;				
		pktOut.id = 0x41;
		pktOut.data = (BYTE *) rbuf;

		if ( bcGPSMan (hBC_PCI, &pktIn, &pktOut) == TRUE ){	
			pac41.time   = getFloat (&rbuf[0]);
			pac41.week   = getShort (&rbuf[4]);
			pac41.offset = getFloat (&rbuf[6]);
			
			printf ("Seconds of Week: %0.2f\n", pac41.time);
			printf ("GPS Week Number: %hd\n", pac41.week);
			printf ("GPS/UCT Offset:  %0.2f\n\n", pac41.offset);
			
			nxt();
		}
		else
			errout();
	}
}

//**************************************************************************************
// Description : GPS Packet 42 - Position Packet
//**************************************************************************************
void getPac42 (BC_PCI_HANDLE hBC_PCI)
{
	GpsPkt pkt;
	
	printf ("\n\nGPS PACKET 42 - GPS POSITION PACKET\n\n");
	
	pkt.id = 0x42;
	pkt.len = 16;
	pkt.data = (BYTE *) rbuf;

	if (bcGPSReq(hBC_PCI, &pkt) == TRUE)
	{
		pac42.X    = getFloat (&rbuf[0]);
		pac42.Y    = getFloat (&rbuf[4]);
		pac42.Z    = getFloat (&rbuf[8]);
		pac42.TofF = getFloat (&rbuf[12]);
		
		printf ("X Coordinate:\t%0.2f meters\n", pac42.X);
		printf ("Y Coordinate:\t%0.2f meters\n", pac42.Y);
		printf ("Z Coordinate:\t%0.2f meters\n", pac42.Z);
		printf ("Time-of-fix: \t%0.2f seconds\n\n", pac42.TofF);
		nxt();
	}
	else
	{
		// Since the reciver may not have the data ready yet,
		// lets give it one more try
		sleep(1);
		pkt.id = 0x42;
		pkt.len = 16;
		pkt.data = (BYTE *) rbuf;
		
		if (bcGPSReq(hBC_PCI, &pkt) == TRUE){
			pac42.X    = getFloat (&rbuf[0]);
			pac42.Y    = getFloat (&rbuf[4]);
			pac42.Z    = getFloat (&rbuf[8]);
			pac42.TofF = getFloat (&rbuf[12]);
			
			printf ("X Coordinate:\t%0.2f\n", pac42.X);
			printf ("Y Coordinate:\t%0.2f\n", pac42.Y);
			printf ("Z Coordinate:\t%0.2f\n", pac42.Z);
			printf ("Time-of-fix: \t%0.2f\n\n", pac42.TofF);
			nxt();
		}
		else
			errout();
	}
		
}

//**************************************************************************************
// Description : GPS Packet 46 - Health Packet
//**************************************************************************************
void getPac46 (BC_PCI_HANDLE hBC_PCI)
{
	GpsPkt pkt;
	
	printf ("\n\nGPS PACKET 46 - GPS HEALTH PACKET\n\n");

	pkt.id = 0x46;
	pkt.len = 2;
	pkt.data = (BYTE *) rbuf;
	
	if (bcGPSReq(hBC_PCI, &pkt) == FALSE)
		sleep(1);

	pkt.id = 0x46;
	pkt.len = 2;
	pkt.data = (BYTE *) rbuf;

	if (bcGPSReq(hBC_PCI, &pkt) == TRUE)
	{
		switch(rbuf[0]){
		case 0: printf("\nStatus: Doing position fixes");
			break;
		case 1: printf("\nStatus: Don't have GPS time yet");
			break;
		case 3: printf("\nStatus: PDOP is too high");
			break;
		case 8: printf("\nStatus: No usable satellites");
			break;
		case 9: printf("\nStatus: Only 1 usable satellite");
			break;
		case 10: printf("\nStatus: Only 2 usable satellite");
			break;
		case 11: printf("\nStatus: Only 3 usable satellite");
			break;
		case 12: printf("\nStatus: The chosen satellite is unusable");
			break;
		default: printf("\nStatus: Unknown");
		}

		printf("\nError: %x", rbuf[1]);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : converts an array of 2 bytes to a short
//**************************************************************************************
short getShort (char *pbDat)
{
	union {
		char    bd[2];
		short   s;
	}  uval;
	
		uval.bd[1] = *(pbDat++);
		uval.bd[0] = *pbDat;
	
	return uval.s;
}


//**************************************************************************************
// Description : converts an array of 4 bytes to a float 
//**************************************************************************************
float getFloat (char *bp)
{
	union {
		char    bd[4];
		float   fv;
	}  uval;
		uval.bd[3] = *(bp++);
		uval.bd[2] = *(bp++);
		uval.bd[1] = *(bp++);
		uval.bd[0] = *bp;
	return uval.fv;
}

//**************************************************************************************
// Description : Read from dual port RAM
//**************************************************************************************
void pci_read_dp_ram (BC_PCI_HANDLE hBC_PCI)
{
	unsigned int dwOffset;
	BYTE  bData;
	printf ("\n\n");
	printf ("Enter hex offset (0x000 - 0x800): 0x");
	scanf ("%x", &dwOffset);
	if (bcReadDPReg (hBC_PCI, dwOffset, &bData) == TRUE)
	{
		printf ("Read DP RAM 0x%03x: 0x%02x\n", dwOffset, (int)bData);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Write to dual port RAM
//**************************************************************************************
void pci_write_dp_ram (BC_PCI_HANDLE hBC_PCI)
{
	unsigned int dwOffset;
	int   iData;
	BYTE  bData;
	printf ("\n\n");
	printf ("Enter hex offset (0x000 - 0x800): 0x");
	scanf ("%x", &dwOffset);
	printf ("Enter hex data (0x00 - 0xFF): 0x");
	scanf ("%x", &iData);
	bData = (BYTE)(iData & 0xFF);
	if (bcWriteDPReg (hBC_PCI, dwOffset, bData) == TRUE)
	{
		printf ("Wrote DP RAM 0x%03x with 0x%02x\n", dwOffset, iData & 0xFF);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Write ACK register to send DP command to timing engine
//**************************************************************************************
void pci_write_ack (BC_PCI_HANDLE hBC_PCI)
{
	DWORD dwData = 0x81;
	if (bcWriteReg (hBC_PCI, PCI_ACK_OFFSET, dwData) == TRUE)
	{
		printf ("Wrote ACK register\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Select periodic output or DDS output
//**************************************************************************************
void dds_set_periodic_dds_select (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	BYTE  bData;
	printf ("\n\n");
	printf ("Enter choice (0 - periodic output, 1 - DDS output): ");
	scanf ("%d", &iVal);
	bData = (BYTE)(iVal & 0xFF);
	if (bcSetPeriodicDDSSelect (hBC_PCI, bData) == TRUE)
	{
		printf ("Done setting periodic/DDS output\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Enable or disable periodic/DDS output
//**************************************************************************************
void dds_set_periodic_dds_enable (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	BYTE  bData;
	printf ("\n\n");
	printf ("Enter choice (0 - disable DDS/periodic output, 1 - enable DDS/periodic output): ");
	scanf ("%d", &iVal);
	bData = (BYTE)(iVal & 0xFF);
	if (bcSetPeriodicDDSEnable (hBC_PCI, bData) == TRUE)
	{
		printf ("Done setting periodic/DDS enable or disable\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS divider value
//**************************************************************************************
void dds_set_dds_divider (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	BYTE  bData;
	printf ("\n\n");
	printf ("Choice (0 - div by 10^0, 1 - div by 10^1, 2 - div by 10^2,\n");
	printf ("        3 - div by 10^3, 4 - div by 10^4, 5 - div by 10^5,\n");
	printf ("        6 - div by 10^6, 7 - div by 10^7,\n");
	printf ("        15 - div by period value in the period register)\n");
	printf ("Enter choice: ");
	scanf ("%d", &iVal);
	bData = (BYTE)(iVal & 0xFF);
	if (bcSetDDSDivider (hBC_PCI, bData) == TRUE)
	{
		printf ("Done setting DDS divider\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS divider source
//**************************************************************************************
void dds_set_dds_divider_source (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	BYTE  bData;
	printf ("\n\n");
	printf ("Enter choice (0 - DDS, 1 - DDS multiplier, 2 - 100MHz): ");
	scanf ("%d", &iVal);
	bData = (BYTE)(iVal & 0xFF);
	if (bcSetDDSDividerSource (hBC_PCI, bData) == TRUE)
	{
		printf ("Done setting DDS divider source\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS sync mode
//**************************************************************************************
void dds_set_dds_sync_mode (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	BYTE  bData;
	printf ("\n\n");
	printf ("Enter choice (0 - fractional mode, 1 - continuous mode): ");
	scanf ("%d", &iVal);
	bData = (BYTE)(iVal & 0xFF);
	if (bcSetDDSSyncMode (hBC_PCI, bData) == TRUE)
	{
		printf ("Done setting DDS sync mode\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS multiplier
//**************************************************************************************
void dds_set_dds_multiplier (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	BYTE  bData;
	printf ("\n\n");
	printf ("Choice (1 - DDS x 1,   2 - DDS x 2,   3 - DDS x 3,\n");
	printf ("        4 - DDS x 4,   6 - DDS x 6,   8 - DDS x 8,\n");
	printf ("        10 - DDS x 10, 16 - DDS x 16)\n");
	printf ("Enter choice: ");
	scanf ("%d", &iVal);
	bData = (BYTE)(iVal & 0xFF);
	if (bcSetDDSMultiplier (hBC_PCI, bData) == TRUE)
	{
		printf ("Done setting DDS multiplier\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS period value
//**************************************************************************************
void dds_set_dds_period_value (BC_PCI_HANDLE hBC_PCI)
{
	int   iVal;
	printf ("\n\n");
	printf ("Enter period value: ");
	scanf ("%d", &iVal);
	if (bcSetDDSPeriodValue (hBC_PCI, (DWORD)iVal) == TRUE)
	{
		printf ("Done setting DDS period value\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS tuning word
//**************************************************************************************
void dds_set_dds_tuning_word (BC_PCI_HANDLE hBC_PCI)
{
	DWORD dwVal;
	printf ("\n\n");
	printf ("Enter tuning word: ");
	scanf ("%ld", &dwVal);
	if (bcSetDDSTuningWord (hBC_PCI, dwVal) == TRUE)
	{
		printf ("Done setting DDS tuning word\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set DDS output frequency
//**************************************************************************************
void dds_set_dds_frequency (BC_PCI_HANDLE hBC_PCI)
{
	double fVal;
	printf ("\n\n");
	printf ("Enter DDS Frequency (in Hz): ");
	scanf ("%lf", &fVal);
	if (bcSetDDSFrequency (hBC_PCI, fVal) == TRUE)
	{
		printf ("Done setting DDS frequency\n");
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Set Time Code Format & Type Extended (V2 Hardware)
//**************************************************************************************
void pci_time_code_ex (BC_PCI_HANDLE hBC_PCI)
{
	int tc_fmt = 0;
	int tc_mod = 0;

	do {
		printf ("\n\nSelect Time Code Format:\n\n");
		printf ("  1. IRIG A000-A003, A130-A133 (no year)\n");
		printf ("  2. IRIG A004-A007, A134-A137 (has year)\n");
		printf ("  3. IRIG B000-B003, B120-B123 (no year)\n");
		printf ("  4. IRIG B004-B007, B124-B127 (has year)\n");
		printf ("  5. IRIG B TrueTime Legacy\n");
		printf ("  6. IRIG B IEEE 1344\n");
		printf ("  7. IRIG E001, E002, E121, E122 (no year)\n");
		printf ("  8. IRIG E005, E006, E125, E126 (has year)\n");
		printf ("  9. IRIG E001, E002, E111, E112 (no year)\n");
		printf (" 10. IRIG E005, E006, E115, E116 (has year)\n");
		printf (" 11. IRIG G001, G002, G141, G142 (no year)\n");
		printf (" 12. IRIG G005, G006, G145, G146 (has year)\n");
		printf (" 13. NASA 36\n");
		printf (" 14. XR3\n");
		printf (" 15. 2137\n");
		printf ("  0. Back to main menu \n\n");
		printf ("     Select: ");
		scanf  ("%d", &tc_fmt);
		switch ( tc_fmt )
		{
			case 1:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 2:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_Y) == FALSE)
					errout();
				break;
			case 3:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 4:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_Y) == FALSE)
					errout();
				break;
			case 5:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_T) == FALSE)
					errout();
				break;
			case 6:
				if (bcSetTcInEx (hBC_PCI, TCODE_IEEE, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 7:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_E, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 8:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_E, TCODE_IRIG_SUBTYPE_Y) == FALSE)
					errout();
				break;
			case 9:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_e, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 10:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_e, TCODE_IRIG_SUBTYPE_Y) == FALSE)
					errout();
				break;
			case 11:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_G, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 12:
				if (bcSetTcInEx (hBC_PCI, TCODE_IRIG_G, TCODE_IRIG_SUBTYPE_Y) == FALSE)
					errout();
				break;
			case 13:
				if (bcSetTcInEx (hBC_PCI, TCODE_NASA, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 14:
				if (bcSetTcInEx (hBC_PCI, TCODE_XR3, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 15:
				if (bcSetTcInEx (hBC_PCI, TCODE_2137, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 0:
			default:
				tc_fmt = 0;
				break;
		}

		if (tc_fmt == 0)
			break;

		printf ("\n\nSelect Time Code Modulation:\n\n");
		printf ("  1. Modulated\n");
		printf ("  2. DC Level Shift\n\n");
		printf ("     Select: ");
		scanf  ("%d", &tc_mod);
		switch ( tc_mod )
		{
			case 1: if (bcSetTcInMod(hBC_PCI,TCODE_MOD_AM) == FALSE)
						errout();			
					break;
			case 2: if (bcSetTcInMod(hBC_PCI,TCODE_MOD_DC) == FALSE)
						errout();		
					break;
			default:usererr();
					break;
		}
	} while (tc_fmt!=0);
}

//**************************************************************************************
// Description : Set TimeCode Output Format Extended (V2 Hardware)
//**************************************************************************************
void pci_tc_out_format_ex(BC_PCI_HANDLE hBC_PCI)
{
	int tc_out = 0;
	
	do {
		printf ("\n\nSelect Time Code Output Format\n\n");
		printf ("  1. IRIG A000, A130 (BCD,CF,SBS)      2. IRIG A001, A131 (BCD,CF)\n");
		printf ("  3. IRIG A002, A132 (BCD)             4. IRIG A003, A133 (BCD,SBS)\n");
		printf ("  5. IRIG A004, A134 (BCD,YR,CF,SBS)   6. IRIG A005, A135 (BCD,YR,CF)\n");
		printf ("  7. IRIG A006, A136 (BCD,YR)          8. IRIG A007, A137 (BCD,YR,SBS)\n");
		printf ("\n");
		printf ("  9. IRIG B000, B120 (BCD,CF,SBS)     10. IRIG B001, B121 (BCD,CF)\n");
		printf (" 11. IRIG B002, B122 (BCD)            12. IRIG B003, B123 (BCD,SBS)\n");
		printf (" 13. IRIG B004, B124 (BCD,YR,CF,SBS)  14. IRIG B005, B125 (BCD,YR,CF)\n");
		printf (" 15. IRIG B006, B126 (BCD,YR)         16. IRIG B007, B127 (BCD,YR,SBS)\n");
		printf (" 17. IRIG B TrueTime (BCD,CF,SBS)     18. IRIG B IEEE 1344\n");
		printf ("\n");
		printf (" 19. IRIG E001, E121 (BCD,CF)         20. IRIG E002, E122 (BCD)\n");
		printf (" 21. IRIG E005, E125 (BCD,YR,CF)      22. IRIG E006, E126 (BCD,YR)\n");
		printf (" 23. IRIG E001, E111 (BCD,CF)         24. IRIG E002, E112 (BCD)\n");
		printf (" 25. IRIG E005, E115 (BCD,YR,CF)      26. IRIG E006, E116 (BCD,YR)\n");
		printf ("\n");
		printf (" 27. IRIG G005, G145 (BCD,YR,CF)      28. NASA 36\n");
		printf (" 29. XR3                              30. 2137\n");
		printf ("  0. Back to main menu \n\n");
		printf ("     Select: ");
		scanf ("%d", &tc_out);

		switch ( tc_out )
		{
			case 1:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_0) == FALSE)
					errout();
				break;
			case 2:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_1) == FALSE)
					errout();
				break;
			case 3:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_2) == FALSE)
					errout();
				break;
			case 4:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_3) == FALSE)
					errout();
				break;
			case 5:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_4) == FALSE)
					errout();
				break;
			case 6:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_5) == FALSE)
					errout();
				break;
			case 7:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_6) == FALSE)
					errout();
				break;
			case 8:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_A, TCODE_IRIG_SUBTYPE_7) == FALSE)
					errout();
				break;
			case 9:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_0) == FALSE)
					errout();
				break;
			case 10:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_1) == FALSE)
					errout();
				break;
			case 11:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_2) == FALSE)
					errout();
				break;
			case 12:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_3) == FALSE)
					errout();
				break;
			case 13:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_4) == FALSE)
					errout();
				break;
			case 14:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_5) == FALSE)
					errout();
				break;
			case 15:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_6) == FALSE)
					errout();
				break;
			case 16:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_7) == FALSE)
					errout();
				break;
			case 17:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_B, TCODE_IRIG_SUBTYPE_T) == FALSE)
					errout();
				break;
			case 18:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IEEE, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 19:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_E, TCODE_IRIG_SUBTYPE_1) == FALSE)
					errout();
				break;
			case 20:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_E, TCODE_IRIG_SUBTYPE_2) == FALSE)
					errout();
				break;
			case 21:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_E, TCODE_IRIG_SUBTYPE_5) == FALSE)
					errout();
				break;
			case 22:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_E, TCODE_IRIG_SUBTYPE_6) == FALSE)
					errout();
				break;
			case 23:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_e, TCODE_IRIG_SUBTYPE_1) == FALSE)
					errout();
				break;
			case 24:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_e, TCODE_IRIG_SUBTYPE_2) == FALSE)
					errout();
				break;
			case 25:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_e, TCODE_IRIG_SUBTYPE_5) == FALSE)
					errout();
				break;
			case 26:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_e, TCODE_IRIG_SUBTYPE_6) == FALSE)
					errout();
				break;
			case 27:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_IRIG_G, TCODE_IRIG_SUBTYPE_5) == FALSE)
					errout();
				break;
			case 28:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_NASA, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 29:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_XR3, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 30:
				if (bcSetGenCodeEx (hBC_PCI, TCODE_2137, TCODE_IRIG_SUBTYPE_NONE) == FALSE)
					errout();
				break;
			case 0:
			default:
				tc_out = 0;
				break;
		}
	} while (tc_out!=0);
}

//**************************************************************************************
// Description : Read from register
//**************************************************************************************
void pci_read_register (BC_PCI_HANDLE hBC_PCI)
{
	unsigned int dwOffset;
	DWORD dwData;
	printf ("\n\n");
	printf ("Enter hex offset (0x00 - 0x80): 0x");
	scanf ("%x", &dwOffset);
	if (bcReadReg (hBC_PCI, dwOffset, &dwData) == TRUE)
	{
		printf ("Read Register 0x%02x: 0x%08x\n", dwOffset, (unsigned int)dwData);
		nxt();
	}
	else
		errout();
}

//**************************************************************************************
// Description : Write to register
//**************************************************************************************
void pci_write_register (BC_PCI_HANDLE hBC_PCI)
{
	unsigned int dwOffset;
	unsigned int dwData;
	printf ("\n\n");
	printf ("Enter hex offset (0x00 - 0x80): 0x");
	scanf ("%x", &dwOffset);
	printf ("Enter hex data: 0x");
	scanf ("%x", &dwData);
	if (bcWriteReg (hBC_PCI, dwOffset, dwData) == TRUE)
	{
		printf ("Wrote Register 0x%02x with 0x%08x\n", dwOffset, dwData);
		nxt();
	}
	else
		errin();
}

//**************************************************************************************
// Description : Set events settings --- special for event2 and event3 support
//**************************************************************************************
void pci_sel_events_settings (BC_PCI_HANDLE hBC_PCI)
{
	EventsData EvDat;
	int iVal;

	printf ("\n\n");
	printf ("Event trigger source (0 - event, 1 - heartbeat): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event trigger source.\n");
		nxt();
		return;
	}
	EvDat.evtsrc = (BYTE)iVal;

	// Select event (or event1) settings
	printf ("Event enable (1 - enable, 0 - disable): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event enable.\n");
		nxt();
		return;
	}
	EvDat.evtctl = (BYTE)iVal;

	printf ("Select event sense (0 - rising, 1 - falling): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event sense.\n");
		nxt();
		return;
	}
	EvDat.evtsense = (BYTE)iVal;

	printf ("Select event capture lock (1 - lock, 0 - not lock): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event capture lock.\n");
		nxt();
		return;
	}
	EvDat.evtlock = (BYTE)iVal;

	// Select event2 settings
	printf ("Event2 enable (1 - enable, 0 - disable): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event2 enable.\n");
		nxt();
		return;
	}
	EvDat.evt2ctl = (BYTE)iVal;

	printf ("Select event2 sense (0 - rising, 1 - falling): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event2 sense.\n");
		nxt();
		return;
	}
	EvDat.evt2sense = (BYTE)iVal;

	printf ("Select event2 capture lock (1 - lock, 0 - not lock): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event2 capture lock.\n");
		nxt();
		return;
	}
	EvDat.evt2lock = (BYTE)iVal;

	// Select event3 settings
	printf ("Event3 enable (1 - enable, 0 - disable): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event3 enable.\n");
		nxt();
		return;
	}
	EvDat.evt3ctl = (BYTE)iVal;

	printf ("Select event3 sense (0 - rising, 1 - falling): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event3 sense.\n");
		nxt();
		return;
	}
	EvDat.evt3sense = (BYTE)iVal;

	printf ("Select event3 capture lock (1 - lock, 0 - not lock): ");
	if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
	{
		printf ("Error: invalid input for event3 capture lock.\n");
		nxt();
		return;
	}
	EvDat.evt3lock = (BYTE)iVal;

	if (bcSetEventsData (hBC_PCI, &EvDat) == TRUE)
	{
		printf ("Done selecting events settings.\n");
		nxt();
	}
	else
		errin();
}

//**************************************************************************************
// Description : GPS Menu
//**************************************************************************************
void gps_menu (BC_PCI_HANDLE hBC_PCI)
{
	int gps_sel;
	
	do{
		printf ("\n\n\n\nGPS Packet Menu\n\n");
		printf (" 1. Request Packet 41 - Gps Time Packet\n");
		printf (" 2. Request Packet 42 - Gps Position Packet\n");
		printf (" 3. Request Packet 46 - Gps Health Packet\n");
		printf (" 0. Back to main menu \n\n");
		printf ("    Select: ");
		scanf ("%d", &gps_sel);

		switch (gps_sel){
		case 1: getPac41(hBC_PCI);
			break;
		case 2: getPac42(hBC_PCI);
			break;
		case 3: getPac46(hBC_PCI);
			break;
		case 0:
		default: gps_sel = 0;
		}
	}while (gps_sel!=0);
}

//**************************************************************************************
// Description : Advanced Menu
//**************************************************************************************
void advanced_main (BC_PCI_HANDLE hBC_PCI)
{
	int adv_menu = 0;

	do{
		printf ("\n\n\n\nAdvanced Menu\n\n");
		printf (" 1. Control Jam-Sync\n");
		printf (" 2. Force Jam-Sync\n");
		printf (" 3. Set Clock Value\n");
		printf (" 4. Load D/A Converter\n");
		printf (" 5. Set Disciplining Gain\n");
		printf (" 6. Disconnect Battery to RTC Chip\n");
		printf (" 0. Back to main menu \n\n");
		printf ("    Select: ");
		scanf ("%d", &adv_menu);

		switch (adv_menu){
		case 1: pci_ctl_jam_sync(hBC_PCI);
			break;
		case 2: pci_frc_jam_sync(hBC_PCI);
			break;
		case 3: pci_set_clk_val(hBC_PCI);
			break;
		case 4: pci_set_da_con(hBC_PCI);
			break;
		case 5: pci_set_gain(hBC_PCI);
			break;
		case 6: pci_dis_rtc(hBC_PCI);
			break;
		case 0:
		default: adv_menu = 0;
		}
	}while (adv_menu!=0);
}

//**************************************************************************************
// Description : Interrupts Menu
//**************************************************************************************
void interrupts_menu (BC_PCI_HANDLE hBC_PCI)
{
	int int_menu = 0;

	do{
		printf ("\n\n\n\nInterrupts Menu\n\n");
		printf (" 1. Set Interrupts to start Interrupt Routine\n");
		printf (" 2. Stop Interrupt Routine\n");
		printf (" 3. Request Interrupts\n");
		printf (" 0. Back to main menu \n\n");
		printf ("    Select: ");
		scanf ("%d", &int_menu);

		switch (int_menu){
		case 1: pci_set_ints(hBC_PCI);
			break;
		case 2: bcStopInt(hBC_PCI); 
			break;
		case 3: pci_req_ints(hBC_PCI);
			break;
		case 0:
		default: int_menu = 0;
			break;
		}
	}while (int_menu!=0);
}

//**************************************************************************************
// Description : DDS Menu
//**************************************************************************************
void dds_menu (BC_PCI_HANDLE hBC_PCI)
{
	int dds_sel;

	do {
		printf ("\n\n\n\nDDS Menu\n\n");
		printf (" 1. Set DDS Frequency\n");
		printf (" 2. Select Periodic or DDS Output\n");
		printf (" 3. Enable/Disable Periodic/DDS Output\n");
		printf (" 4. Set DDS Divider\n");
		printf (" 5. Set DDS Divider Source\n");
		printf (" 6. Set DDS Sync Mode\n");
		printf (" 7. Set DDS Multiplier\n");
		printf (" 8. Set DDS Period Value\n");
		printf (" 9. Set DDS Tuning Word\n");
		printf (" 0. Back to main menu \n\n");
		printf ("    Select: ");
		scanf ("%d", &dds_sel);

		switch (dds_sel) {
		case 1: dds_set_dds_frequency (hBC_PCI);
			break;
		case 2: dds_set_periodic_dds_select (hBC_PCI);
			break;
		case 3: dds_set_periodic_dds_enable (hBC_PCI);
			break;
		case 4: dds_set_dds_divider (hBC_PCI);
			break;
		case 5: dds_set_dds_divider_source (hBC_PCI);
			break;
		case 6: dds_set_dds_sync_mode (hBC_PCI);
			break;
		case 7: dds_set_dds_multiplier (hBC_PCI);
			break;
		case 8: dds_set_dds_period_value (hBC_PCI);
			break;
		case 9: dds_set_dds_tuning_word (hBC_PCI);
			break;
		case 0:
		default: dds_sel = 0;
		}
	} while (dds_sel!=0);
}

//**************************************************************************************
// Description : Main Menu
//**************************************************************************************
int demo_menu(BC_PCI_HANDLE hBC_PCI)
{
	int cmd = 0;

	do{
		printf ("\n\n\n--------------------------------------------------------------------------------\n");
		printf ("                             Symmetricom - TT & M\n");
		printf ("                         bc635/637PCIe-V2 Configurator               Version 3.0\n");
		printf ("--------------------------------------------------------------------------------\n");
		printf ("  1. Read Current Time (Press enter to return to menu)\n");
		printf ("  2. Read Event Time   (Press enter to return to menu)\n");
		printf ("  3. Set Current Time \n");
		printf ("  4. Set Current Year\n");
		printf ("  5. Set Strobe Time\n");
		printf ("  6. Program Control Register\n");
		printf ("  7. Program Leap Event Seconds\n");
		printf ("  8. Select Time Format\n");
		printf ("  9. Select Operational Mode\n");
		printf (" 10. Select Decoding TimeCode Format\n");
		printf (" 11. Select TimeCode Output Format\n");
		printf (" 12. Select Clock Source\n");
		printf (" 13. Select Output Frequency\n");
		printf (" 14. Program Heartbeat counters\n");
		printf (" 15. Set Local Time Offset         \t\t16. Set Generator Time Offset\n");
		printf (" 17. Set Propagation Delay         \t\t18. Set Local Time Offset Flag\n");
		printf (" 19. Set Year Auto Increment Flag  \t\t20. Sync RTC to External Time Data\n");
		printf (" 21. Set GPS Time Format           \t\t22. Set GPS Mode Flag\n");
		printf (" 23. Software Reset                \t\t24. Request Time Settings\n");
		printf (" 25. Request Time Code Settings    \t\t26. Request Clock Settings\n");
		printf (" 27. Request Control Settings      \t\t28. Request Model Information\n");
		printf (" 29. Request Firmware Version      \t\t30. Advanced Menu\n");
		printf (" 31. Interrupts Menu               \t\t32. GPS Menu - bc637PCI Only\n");
		printf (" 33. PCI Revision ID               \t\t34. Read from Dual Port RAM\n");
		printf (" 35. Write to Dual Port RAM        \t\t36. Write ACK - Send DP Command\n");
		printf (" 37. DDS Menu - V2 Hardware Only\n");
		printf (" 38. Select TimeCode Input Format - V2 Hardware\n");
		printf (" 39. Select TimeCode Output Format - V2 Hardware\n");
		printf (" 40. Read from Register            \t\t41. Write to Register\n");
		printf (" 42. Select events settings\n");
		printf (" 43. Read Event2 Time  (Press enter to return to menu)\n");
		printf (" 44. Read Event3 Time  (Press enter to return to menu)\n");
		printf (" 0.  Exit the Program...........................................................\n\n");
		printf ("     Select Option: ");
		scanf ("%d",&cmd);

		switch(cmd){
			case 1 : pci_read_time(hBC_PCI);
					break;
			case 2 : pci_read_event_time(hBC_PCI);
					break;
			case 3 : pci_set_time (hBC_PCI);
					break;
			case 4 : pci_set_year(hBC_PCI);
					break;
			case 5 : pci_set_strobe(hBC_PCI);
					break;
			case 6 : pci_set_control(hBC_PCI);
					break;
			case 7 : pci_set_leap_sec (hBC_PCI);
					break;
			case 8 : pci_time_format(hBC_PCI);
					break;
			case 9 : pci_mode(hBC_PCI);
					break;
			case 10 : pci_time_code(hBC_PCI);
					break;
			case 11 : pci_tc_out_format(hBC_PCI); 
					break;
			case 12 : pci_set_clock_src(hBC_PCI);
					break;
			case 13 : pci_out_freq(hBC_PCI); 
					break;
			case 14: pci_heartbeat(hBC_PCI);
					break;
			case 15: pci_set_loc_off (hBC_PCI);
					break;
			case 16: pci_set_gen_off (hBC_PCI);
					break;
			case 17: pci_set_prop_delay (hBC_PCI);
					break;
			case 18 : pci_set_local_off_flg(hBC_PCI);
					break;
			case 19 : pci_set_yr_auto_inc_flg(hBC_PCI);
					break;
			case 20 : pci_sync_rtc(hBC_PCI);
					break;
			case 21 : pci_set_gps_tm_fmt(hBC_PCI);
					break;
			case 22 : pci_set_gps_mode_flg(hBC_PCI);
					break;
			case 23 : pci_sw_rest (hBC_PCI);
					break;
			case 24 : pci_req_time_settings(hBC_PCI);
					break;
			case 25 : pci_req_timecode_settings (hBC_PCI);
					break;
			case 26 : pci_req_clock_settings (hBC_PCI);
					break;
			case 27 : pci_req_control_settings (hBC_PCI);
					break;
			case 28: pci_req_modal_data (hBC_PCI);
					break;
			case 29: pci_req_fw_ver (hBC_PCI);
					break;
			case 30: advanced_main (hBC_PCI);
					break;
			case 31: interrupts_menu (hBC_PCI);
					break;
			case 32: gps_menu (hBC_PCI);
					break;
			case 33: pci_req_rev_id (hBC_PCI);
					break;
			case 34: pci_read_dp_ram (hBC_PCI);
					break;
			case 35: pci_write_dp_ram (hBC_PCI);
					break;
			case 36: pci_write_ack (hBC_PCI);
					break;
			case 37: dds_menu (hBC_PCI);
					break;
			case 38: pci_time_code_ex (hBC_PCI);
					break;
			case 39 : pci_tc_out_format_ex (hBC_PCI); 
					break;
			case 40: pci_read_register (hBC_PCI);
					break;
			case 41: pci_write_register (hBC_PCI);
					break;
			case 42 : pci_sel_events_settings(hBC_PCI);
					break;
			case 43 : pci_read_event2_time(hBC_PCI);
					break;
			case 44 : pci_read_event3_time(hBC_PCI);
					break;
			case 0  : 
			default : cmd = 0;
					  printf("\n\nexiting......\n\n");
					  break;
		}
	} while (cmd!=0);

	return 0;
}

//**************************************************************************************
// Description : Wait for user to press return
//**************************************************************************************
int nxt ()
{
	getchar();
	printf("\n\n>>>>>>>>>>>>>>>>  Press Enter to continue  <<<<<<<<<<<<<<<<<\n");
	getchar(); 
	return 0;
}

//**************************************************************************************
// Description : User selection error
//**************************************************************************************
int usererr ()
{
	printf ("\nError: Unrecognized Selection!!!\n");
	nxt();
	return 0;
}

//**************************************************************************************
// Description : Error getting data from the hardware
//**************************************************************************************
int errout ()
{
	printf("\n\nError: Getting data!!!!!!!!!!!!!!!!!!!!!!\n\n");
	nxt();
	return 0;
}

//**************************************************************************************
// Description : Error sending data to the hardware
//**************************************************************************************
int errin ()
{
	printf("\n\nError: Sending data!!!!!!!!!!!!!!!!!!!!!!\n\n");
	nxt();
	return 0;
}

//**************************************************************************************
// Description : Interrupt Error
//**************************************************************************************
int interrupterr ()
{
	printf("\n\nError Setting Interrupt!\n");
	nxt();
	return 0;
}

//**************************************************************************************
// Description : MS-DOS kbhit() in Linux
//**************************************************************************************
int kbhit(void)
{
  struct timeval tv;
  fd_set read_fd;

  tv.tv_sec=0;
  tv.tv_usec=0;

  FD_ZERO(&read_fd);
  FD_SET(0,&read_fd);

  if(select(1, &read_fd, NULL, NULL, &tv) == -1)
		return 0;	

  if(FD_ISSET(0,&read_fd))
		return 1;

  return 0;
}


